var HTTPMethod;
(function (HTTPMethod) {
    HTTPMethod["Delete"] = "delete";
    HTTPMethod["Get"] = "get";
    HTTPMethod["Put"] = "put";
})(HTTPMethod || (HTTPMethod = {}));
var ResponseType;
(function (ResponseType) {
    ResponseType["ArrayBuffer"] = "arrayBuffer";
    ResponseType["Blob"] = "blob";
    ResponseType["JSON"] = "json";
    ResponseType["Stream"] = "stream";
    ResponseType["Text"] = "text";
})(ResponseType || (ResponseType = {}));
const EXPIRY_HEADER = 'x-nf-expires-at';
export class Blobs {
    constructor({ authentication, context, fetcher, siteID }) {
        this.context = context ?? 'production';
        this.fetcher = fetcher ?? globalThis.fetch;
        this.siteID = siteID;
        if ('contextURL' in authentication) {
            this.authentication = authentication;
        }
        else {
            this.authentication = {
                apiURL: authentication.apiURL ?? 'https://api.netlify.com',
                token: authentication.token,
            };
        }
        if (fetcher) {
            this.fetcher = fetcher;
        }
        else if (globalThis.fetch) {
            this.fetcher = globalThis.fetch;
        }
        else {
            throw new Error('You must specify a fetch-compatible `fetcher` parameter when `fetch` is not available globally');
        }
    }
    async getFinalRequest(key, method) {
        if ('contextURL' in this.authentication) {
            return {
                headers: {
                    authorization: `Bearer ${this.authentication.token}`,
                },
                url: `${this.authentication.contextURL}/${this.siteID}/${this.context}/${key}`,
            };
        }
        const apiURL = `${this.authentication.apiURL}/api/v1/sites/${this.siteID}/blobs/${key}?context=${this.context}`;
        const headers = { authorization: `Bearer ${this.authentication.token}` };
        const res = await this.fetcher(apiURL, { headers, method });
        if (res.status !== 200) {
            throw new Error(`${method} operation has failed: API returned a ${res.status} response`);
        }
        const { url } = await res.json();
        return {
            url,
        };
    }
    static getTTLHeaders(ttl) {
        if (typeof ttl === 'number') {
            return {
                [EXPIRY_HEADER]: (Date.now() + ttl).toString(),
            };
        }
        if (ttl instanceof Date) {
            return {
                [EXPIRY_HEADER]: ttl.getTime().toString(),
            };
        }
        if (ttl === undefined) {
            return {};
        }
        throw new TypeError(`'ttl' value must be a number or a Date, ${typeof ttl} found.`);
    }
    isConfigured() {
        return Boolean(this.authentication?.token) && Boolean(this.siteID);
    }
    async makeStoreRequest(key, method, extraHeaders, body) {
        if (!this.isConfigured()) {
            throw new Error("The blob store is unavailable because it's missing required configuration properties");
        }
        const { headers: baseHeaders = {}, url } = await this.getFinalRequest(key, method);
        const headers = {
            ...baseHeaders,
            ...extraHeaders,
        };
        if (method === HTTPMethod.Put) {
            headers['cache-control'] = 'max-age=0, stale-while-revalidate=60';
        }
        const res = await this.fetcher(url, { body, headers, method });
        if (res.status === 404 && method === HTTPMethod.Get) {
            return null;
        }
        if (res.status !== 200) {
            throw new Error(`${method} operation has failed: store returned a ${res.status} response`);
        }
        return res;
    }
    async delete(key) {
        await this.makeStoreRequest(key, HTTPMethod.Delete);
    }
    async get(key, options) {
        const { type } = options ?? {};
        const res = await this.makeStoreRequest(key, HTTPMethod.Get);
        const expiry = res?.headers.get(EXPIRY_HEADER);
        if (typeof expiry === 'string') {
            const expiryTS = Number.parseInt(expiry);
            if (!Number.isNaN(expiryTS) && expiryTS <= Date.now()) {
                return null;
            }
        }
        if (res === null) {
            return res;
        }
        if (type === undefined || type === ResponseType.Text) {
            return res.text();
        }
        if (type === ResponseType.ArrayBuffer) {
            return res.arrayBuffer();
        }
        if (type === ResponseType.Blob) {
            return res.blob();
        }
        if (type === ResponseType.JSON) {
            return res.json();
        }
        if (type === ResponseType.Stream) {
            return res.body;
        }
        throw new Error(`Invalid 'type' property: ${type}. Expected: arrayBuffer, blob, json, stream, or text.`);
    }
    async set(key, data, { ttl } = {}) {
        const headers = Blobs.getTTLHeaders(ttl);
        await this.makeStoreRequest(key, HTTPMethod.Put, headers, data);
    }
    async setJSON(key, data, { ttl } = {}) {
        const payload = JSON.stringify(data);
        const headers = {
            ...Blobs.getTTLHeaders(ttl),
            'content-type': 'application/json',
        };
        await this.makeStoreRequest(key, HTTPMethod.Put, headers, payload);
    }
}
